﻿using System;
using System.Collections.Generic;
using System.Xml;
using UnityEngine;

namespace MouseController
{
    public class MCInterfaceManager
    {
        #region members


        // list of all known interfaces.
        Dictionary<string, MCInterface> mMCInterfaces = new Dictionary<string, MCInterface>();


        #endregion

        #region Properties

        // the selected interface
        public MCInterface SelectedInterface { get; set; }

        // position of the settings window.
        public Rect SettingsWindowPos { get; set; }

        public List<string> ErrorList { get; set; }

        #endregion


        public MCInterfaceManager()
        {
            SettingsWindowPos = new Rect();
            ErrorList = new List<string>();
        }


        public void LoadSettings(string path)
        {
            mMCInterfaces.Clear();

            XmlDocument doc = new XmlDocument();
            try
            {
                doc.Load(path.Replace("/", "\\"));
                var i = doc.GetElementsByTagName(Consts.cInterface);
                foreach (XmlNode node in i)
                {
                    if (node.Name == Consts.cInterface)
                    {
                        // read interface
                        try
                        {
                            MCInterface newInterface = GetInterface(node);
                            if (!newInterface.IsEmpty)
                                mMCInterfaces.Add(newInterface.Name, newInterface);

                            // read sub interfaces
                            foreach (XmlNode child in node.ChildNodes)
                            {
                                if (child.Name == Consts.cSubInterface)
                                {
                                    MCInterface newSubinterface = GetInterface(child);
                                    if (!newSubinterface.IsEmpty)
                                    {
                                        newSubinterface.Parent = newInterface;
                                        newInterface.Subinterfaces.Add(newSubinterface.Name, newSubinterface);
                                    }
                                }
                                else if (child.Name == Consts.cColorActionMapping)
                                {
                                    if (newInterface == null)
                                        continue;

                                    // read color action mapping
                                    foreach (XmlNode colorActionNode in child.ChildNodes)
                                    {
                                        if (colorActionNode.Name == Consts.cColorAction)
                                        {
                                            MCColorAction colorAction = new MCColorAction();
                                            foreach (XmlAttribute att in colorActionNode.Attributes)
                                            {
                                                if (att.Name == Consts.cName)
                                                    colorAction.Name = att.Value.ToString();

                                                else if (att.Name == Consts.cColor)
                                                    colorAction.Color = GetColor(att.Value.ToString().Trim());

                                                else if (att.Name == Consts.cAction)
                                                    colorAction.Action = GetAction(att.Value.ToString().Trim());
                                            }

                                            //newInterface.ColorActionMapping.Add(colorAction.Action, colorAction);
                                            newInterface.ColorActionMapping.Add(colorAction);
                                        }
                                    }
                                }
                            }
                        }
                        catch (Exception ex)
                        {
                            ErrorList.Add("ErrorMessage: " + ex.Message);
                        }
                    }
                }

                // get last selected interface
                XmlNodeList nodes = doc.GetElementsByTagName(Consts.cSelectedInterface);
                if (nodes.Count == 1)
                {
                    foreach (XmlAttribute att in nodes[0].Attributes)
                    {
                        if (att.Name == Consts.cName)
                        {
                            SetInterface(att.Value.ToString());
                            break;
                        }
                    }
                }

                // select first interface if no last selected interface was found.
                else
                {
                    if (mMCInterfaces.Count > 0)
                    {
                        foreach (KeyValuePair<string, MCInterface> pair in mMCInterfaces)
                        {
                            SetInterface(pair.Key);
                            break;
                        }
                    }
                }

            }
            catch (Exception ex)
            {
                ErrorList.Add("ErrorMessage: " + ex.Message);
            }
        }

        public void SetInterface(string interfaceName)
        {
            if (mMCInterfaces.ContainsKey(interfaceName))
            {
                SelectedInterface = mMCInterfaces[interfaceName];
                SelectedInterface.Initialize();

                //if (mSelectedInterface.TextureMC_Interface == null)
                //{
                //    mSelectedInterface.TextureMC_Interface = MouseControllerPlugin.LoadTexture(MouseControllerPlugin.PluginPath + "/" + 
                //        mSelectedInterface.PicPath.Replace("\\", "/"), mSelectedInterface.Size);
                //    //WWW temp = new WWW(MouseControllerPlugin.PluginPath + "/" + mSelectedInterface.PicPath.Replace("\\", "/"));
                //    //mSelectedInterface.TextureMC_Interface = new Texture2D(mSelectedInterface.Size.Width, mSelectedInterface.Size.Height);
                //    //temp.LoadImageIntoTexture(mSelectedInterface.TextureMC_Interface);
                //}
                //if (mSelectedInterface.TextureMC_Interface_HotSpots == null)
                //{
                //    mSelectedInterface.TextureMC_Interface_HotSpots = MouseControllerPlugin.LoadTexture(MouseControllerPlugin.PluginPath + "/" + 
                //        mSelectedInterface.HotSpotPicPath.Replace("\\", "/"), mSelectedInterface.Size);
                //    //WWW temp = new WWW(MouseControllerPlugin.PluginPath + "/" + mSelectedInterface.HotSpotPicPath.Replace("\\", "/"));
                //    //mSelectedInterface.TextureMC_Interface_HotSpots = new Texture2D(mSelectedInterface.Size.Width, mSelectedInterface.Size.Height);
                //    //temp.LoadImageIntoTexture(mSelectedInterface.TextureMC_Interface_HotSpots);
                //}

                ////mAnalogCenter = GetColorPosition(mTextureMC_Interface_HotSpots, mColorJoystickCenter);

                //mSelectedInterface.CalculateMainInterfacePos();

                //// hook to vessel control
                ////FlightGlobals.ActiveVessel.OnFlyByWire += mSelectedInterface.UpdateControl;

                //foreach (MCInterface entry in mSelectedInterface.Subinterfaces.Values)
                //{
                //    //   entry.Initialize();
                //}
            }
        }

        public void InitializeInterface()
        {
            if (SelectedInterface != null)
                SelectedInterface.Initialize();

            SettingsWindowPos = new Rect(20, 20, 300, 400);
        }

        public void UpdateInterface()
        {
            if (SelectedInterface != null)
            {
                mSelectedInterface.Update();

                // get/adjust mouse position (0, 0 = bottom left corner)
                Vector2 mousePosition = Input.mousePosition;
                mousePosition.y = Camera.current.GetScreenHeight() - Input.mousePosition.y;

                if (SelectedInterface.Collapsed)
                    SelectedInterface.InterfacePos = SelectedInterface.InterfacePosCollapsed;
                else
                    SelectedInterface.InterfacePos = SelectedInterface.InterfacePosExpended;

                //DrawInterface();
                //GUI.DrawTexture(mSelectedInterface.InterfacePos, mSelectedInterface.TextureMC_Interface);

                //foreach (MCInterface entry in mSelectedInterface.Subinterfaces.Values)
                //    entry.Update();

                // draw settings window
                //if (SelectedInterface.ShowSettingsWindow)
                //{
                //    GUI.skin = HighLogic.Skin;
                //    SettingsWindowPos = GUILayout.Window(44834, SettingsWindowPos, DrawWindow, "MouseController - Debug Window", GUILayout.MinWidth(200));
                //}
            }
        }


        private void DrawWindow(int winID)
        {
            switch (winID)
            {
                case 44834: // Debug Window
                    GUILayout.BeginVertical();
                    //GUILayout.Label("Root = " + Root);
                    //GUILayout.Label("PluginFolder = " + PluginFolder);
                    //GUILayout.Label("ButtonPicPath = " + string.Format("file://{0}/Data/MouseController_Button.png", PluginFolder));
                    //GUILayout.Label("InterfacePicPath = " + string.Format("file://{0}/Data/MouseController_Interface.jpg", PluginFolder));
                    //GUILayout.Label("Screen size = (" + mScreenWidth + ", " + mScreenHeight + ")");
                    //GUILayout.Label("MousePosition = (" + mMousePosition.x + ", " + mMousePosition.y + ")");
                    //GUILayout.Label("WindowPosition = (" + mWindowPosition.x + ", " + mWindowPosition.y + ", " + mWindowPosition.width + ", " + mWindowPosition.height + ")");
                    //GUILayout.Label("WindowPosition.Contains = (" + mWindowPosition.Contains(mMousePosition).ToString() + ")");
                    //GUILayout.Label("L Mouse Button = (" + Input.GetMouseButton(0).ToString() + ")");
                    //GUILayout.Label("Rel. Pic Pos = (" + ((int)mMousePosition.x - (int)mWindowPosition.x).ToString() + ", " + ((int)mMousePosition.y - (int)mWindowPosition.y).ToString() + ")");
                    //GUILayout.Label("Texture Pos = (" + ((int)mMousePosition.x - (int)mWindowPosition.x).ToString() + ", " + (mInterfaceHeight - ((int)mMousePosition.y - (int)mWindowPosition.y)).ToString() + ")");
                    //GUILayout.Label("Color = (" + (mCurrentInterfaceStates.Color.r).ToString() + ", " + (mCurrentInterfaceStates.Color.g).ToString() + ", " + (mCurrentInterfaceStates.Color.b).ToString() + ", " + (mCurrentInterfaceStates.Color.a).ToString() + ")");
                    //GUILayout.Label("InterfaceStates:");
                    //GUILayout.Label("ColorArrow = (" + (mColorArrow.r).ToString() + ", " + (mColorArrow.g).ToString() + ", " + (mColorArrow.b).ToString() + ", " + (mColorArrow.a).ToString() + ")");
                    //GUILayout.Label("Arrow (isSameColor)= " + IsSameColor(mCurrentInterfaceStates.Color, mColorArrow).ToString());
                    //GUILayout.Label("HasInput = " + mCurrentInterfaceStates.HasInput.ToString());
                    //GUILayout.Label("Arrow = " + mCurrentInterfaceStates.ArrowDown.ToString());
                    //GUILayout.Label("Func0 = " + mCurrentInterfaceStates.Func0Down.ToString());
                    //GUILayout.Label("Func1 = " + mCurrentInterfaceStates.Func1Down.ToString());
                    //GUILayout.Label("Func2 = " + mCurrentInterfaceStates.Func2Down.ToString());
                    //GUILayout.Label("Func3 = " + mCurrentInterfaceStates.Func3Down.ToString());
                    //GUILayout.Label("Yaw = " + mCurrentInterfaceStates.Yaw.ToString());
                    //GUILayout.Label("Pitch = " + mCurrentInterfaceStates.Pitch.ToString());
                    //GUILayout.Label("Roll = " + mCurrentInterfaceStates.Roll.ToString());
                    //GUILayout.Label("RCS = " + mCurrentInterfaceStates.RCSDown.ToString());
                    //GUILayout.Label("SAS = " + mCurrentInterfaceStates.SASDown.ToString());
                    //GUILayout.Label("NextStage = " + mCurrentInterfaceStates.NextStageDown.ToString());
                    //GUILayout.Label("ThrottleFast = " + mCurrentInterfaceStates.ThrottleMax.ToString());
                    //GUILayout.Label("Throttle = " + mCurrentInterfaceStates.Throttle.ToString());
                    //GUILayout.Label("ThrottleOff = " + mCurrentInterfaceStates.ThrottleOff.ToString());
                    //GUILayout.Label("Actiongroup0 = " + mCurrentInterfaceStates.ActionGroup0Down.ToString());
                    //GUILayout.Label("Actiongroup1 = " + mCurrentInterfaceStates.ActionGroup1Down.ToString());
                    //GUILayout.Label("Actiongroup2 = " + mCurrentInterfaceStates.ActionGroup2Down.ToString());
                    //GUILayout.Label("Actiongroup3 = " + mCurrentInterfaceStates.ActionGroup3Down.ToString());
                    //GUILayout.Label("Actiongroup4 = " + mCurrentInterfaceStates.ActionGroup4Down.ToString());
                    //GUILayout.Label("Actiongroup5 = " + mCurrentInterfaceStates.ActionGroup5Down.ToString());
                    //GUILayout.Label("Actiongroup6 = " + mCurrentInterfaceStates.ActionGroup6Down.ToString());
                    //GUILayout.Label("Actiongroup7 = " + mCurrentInterfaceStates.ActionGroup7Down.ToString());
                    //GUILayout.Label("Actiongroup8 = " + mCurrentInterfaceStates.ActionGroup8Down.ToString());
                    //GUILayout.Label("Actiongroup9 = " + mCurrentInterfaceStates.ActionGroup9Down.ToString());
                    GUILayout.EndVertical();

                    GUI.DragWindow();

                    break;
            }
        }

        private MCInterface GetInterface(XmlNode node)
        {
            MCInterface newInterface = new MCInterface();

            string posX = string.Empty;
            string posY = string.Empty;
            MCSize size = new MCSize();

            foreach (XmlAttribute att in node.Attributes)
            {
                if (att.Name == Consts.cName)
                    newInterface.Name = att.Value.ToString();

                else if (att.Name == Consts.cAlignment)
                    newInterface.Alignment = GetAlignment(att.Value.ToString().Trim());

                else if (att.Name == Consts.cMargin)
                    newInterface.Margin = GetMargin(att.Value.ToString().Trim());

                else if (att.Name == Consts.cX)
                    posX = att.Value;

                else if (att.Name == Consts.cY)
                    posY = att.Value;

                else if (att.Name == Consts.cOpenCloseButtonHeight)
                    newInterface.OpenCloseBtnHeight = int.Parse(att.Value);

                else if (att.Name == Consts.cWidth)
                    size.Width = int.Parse(att.Value);

                else if (att.Name == Consts.cHeight)
                    size.Height = int.Parse(att.Value);

                else if (att.Name == Consts.cPicPath)
                    newInterface.PicPath = att.Value.ToString();

                else if (att.Name == Consts.cHotSpotPicPath)
                    newInterface.HotSpotPicPath = att.Value.ToString();
            }

            Vector2 pos = new Vector2();
            if (!string.IsNullOrEmpty(posX))
                pos.x = ParseInt(posX);
            if (!string.IsNullOrEmpty(posY))
                pos.y = ParseInt(posY);

            newInterface.Position = pos;
            newInterface.Size = size;

            return newInterface;
        }

        private Color GetColor(string colorAsString)
        {
            int r = 0;
            int g = 0;
            int b = 0;

            string[] temp = colorAsString.Split(',');

            if (temp == null || temp.Length != 3)
            {
                //print("MouseController Error: Can't parse Color");
            }
            else
            {
                try
                {
                    r = int.Parse(temp[0].Trim());
                    g = int.Parse(temp[1].Trim());
                    b = int.Parse(temp[2].Trim());
                }
                catch
                {
                    //print("MouseController Error: Can't parse Color");
                }
            }

            return new Color(r, g, b);
        }

        private MCActions GetAction(string actionAsString)
        {
            MCActions action = MCActions.None;

            switch (actionAsString)
            {
                case Consts.cSettings:
                    action = MCActions.Settings;
                    break;
                case Consts.cOpenClose:
                    action = MCActions.OpenClose;
                    break;
                case Consts.cNextVessel:
                    action = MCActions.NextVessel;
                    break;
                case Consts.cPrevVessel:
                    action = MCActions.PrevVessel;
                    break;
                case Consts.cThrottleMax:
                    action = MCActions.ThrottleMax;
                    break;
                case Consts.cThrottleUp:
                    action = MCActions.ThrottleUp;
                    break;
                case Consts.cThrottleDown:
                    action = MCActions.ThrottleDown;
                    break;
                case Consts.cThrottleOff:
                    action = MCActions.ThrottleOff;
                    break;
                case Consts.cRCS:
                    action = MCActions.RCS;
                    break;
                case Consts.cSAS:
                    action = MCActions.SAS;
                    break;
                case Consts.cYawUp:
                    action = MCActions.YawUp;
                    break;
                case Consts.cYawDown:
                    action = MCActions.YawDown;
                    break;
                case Consts.cPitchUp:
                    action = MCActions.PitchUp;
                    break;
                case Consts.cPitchDown:
                    action = MCActions.PitchDown;
                    break;
                case Consts.cRollUp:
                    action = MCActions.RollUp;
                    break;
                case Consts.cRollDown:
                    action = MCActions.RollDown;
                    break;
                case Consts.cNextStage:
                    action = MCActions.NextStage;
                    break;
                case Consts.cActiongroup0:
                    action = MCActions.Actiongroup0;
                    break;
                case Consts.cActiongroup1:
                    action = MCActions.Actiongroup1;
                    break;
                case Consts.cActiongroup2:
                    action = MCActions.Actiongroup2;
                    break;
                case Consts.cActiongroup3:
                    action = MCActions.Actiongroup3;
                    break;
                case Consts.cActiongroup4:
                    action = MCActions.Actiongroup4;
                    break;
                case Consts.cActiongroup5:
                    action = MCActions.Actiongroup5;
                    break;
                case Consts.cActiongroup6:
                    action = MCActions.Actiongroup6;
                    break;
                case Consts.cActiongroup7:
                    action = MCActions.Actiongroup7;
                    break;
                case Consts.cActiongroup8:
                    action = MCActions.Actiongroup8;
                    break;
                case Consts.cActiongroup9:
                    action = MCActions.Actiongroup9;
                    break;
                case Consts.cAnalogCenter:
                    action = MCActions.AnalogCenter;
                    break;
                case Consts.cAnalogUp:
                    action = MCActions.AnalogUp;
                    break;
                case Consts.cAnalogDown:
                    action = MCActions.AnalogDown;
                    break;
                case Consts.cAnalogLeft:
                    action = MCActions.AnalogLeft;
                    break;
                case Consts.cAnalogRight:
                    action = MCActions.AnalogRight;
                    break;
                case Consts.cAnalogUpLeft:
                    action = MCActions.AnalogUpLeft;
                    break;
                case Consts.cAnalogUpRight:
                    action = MCActions.AnalogUpRight;
                    break;
                case Consts.cAnalogDownLeft:
                    action = MCActions.AnalogDownLeft;
                    break;
                case Consts.cAnalogDownRight:
                    action = MCActions.AnalogDownRight;
                    break;
                default:
                    action = MCActions.None;
                    break;
            }

            return action;
        }

        private MCAlignment GetAlignment(string alignmentAsString)
        {
            MCAlignment alignment = MCAlignment.Bottom;

            switch (alignmentAsString)
            {
                case Consts.cLeft:
                    alignment = MCAlignment.Left;
                    break;
                case Consts.cRight:
                    alignment = MCAlignment.Right;
                    break;
                case Consts.cTop:
                    alignment = MCAlignment.Top;
                    break;
                case Consts.cBottom:
                    alignment = MCAlignment.Bottom;
                    break;
            }

            return alignment;
        }

        private MCMargin GetMargin(string marginAsString)
        {
            int l = 0;
            int r = 0;
            int t = 0;
            int b = 0;

            string[] temp = marginAsString.Split(',');

            if (temp == null || temp.Length != 4)
            {
                //print("MouseController Error: Can't parse Margin");
            }
            else
            {
                try
                {
                    l = int.Parse(temp[0].Trim());
                    r = int.Parse(temp[1].Trim());
                    t = int.Parse(temp[2].Trim());
                    b = int.Parse(temp[3].Trim());
                }
                catch
                {
                    //print("MouseController Error: Can't parse Margin");
                }
            }

            return new MCMargin(l, r, t, b);
        }

        private int ParseInt(string integer)
        {
            int pos = 0;
            try
            {
                if (integer.Contains("%"))
                    pos = int.Parse(integer.Replace("%", "").Trim());
                else
                    pos = int.Parse(integer);
            }
            catch
            {
                //print("MouseController error: Error during int parsing!");
            }

            return pos;
        }
    }
}
