﻿using UnityEngine;

namespace MouseController
{
    #region Enumeration

    public enum MCDirections
    {
        None,
        Up,
        Down,
        Left,
        Right

    }

    public enum MCActions
    {
        None,
        Settings,
        OpenClose,
        NextVessel,
        PrevVessel,
        ThrottleMax,
        ThrottleUp,
        ThrottleDown,
        ThrottleOff,
        RCS,
        SAS,
        PitchUp,
        PitchDown,
        YawRight,
        YawLeft,
        RollRight,
        RollLeft,
        NextStage,
        Actiongroup0,
        Actiongroup1,
        Actiongroup2,
        Actiongroup3,
        Actiongroup4,
        Actiongroup5,
        Actiongroup6,
        Actiongroup7,
        Actiongroup8,
        Actiongroup9,
        AnalogCenter,
        AnalogUp,
        AnalogDown,
        AnalogLeft,
        AnalogRight,
        AnalogUpLeft,
        AnalogUpRight,
        AnalogDownLeft,
        AnalogDownRight,
        Func0,
        Func1,
        Func2,
        Func3,
        RCSPitchUp,
        RCSPitchDown,
        RCSYawRight,
        RCSYawLeft,
        RCSRollRight,
        RCSRollLeft,
        RCSAnalogCenter,
        RCSAnalogUp,
        RCSAnalogDown,
        RCSAnalogLeft,
        RCSAnalogRight,
        RCSAnalogUpLeft,
        RCSAnalogUpRight,
        RCSAnalogDownLeft,
        RCSAnalogDownRight,
    }

    public enum MCAlignment
    {
        Left,
        Right,
        Top,
        Bottom
    }

    #endregion

    public class Consts
    {
        #region Constants

        // XML Node names
        public const string cMouseController = "MouseController";
        //public const string cSettings = "Settings"; // see MC Actions
        public const string cSelectedInterface = "SelectedInterface";
        public const string cInterfaces = "Interfaces";
        public const string cInterface = "Interface";
        public const string cSubInterface = "SubInterface";
        public const string cColorActionMapping = "ColorActionMapping";
        public const string cColorAction = "ColorAction";

        // XML Attribute names
        public const string cName = "Name";
        public const string cAlignment = "Alignment";
        public const string cMargin = "Margin";
        public const string cX = "X";
        public const string cY = "Y";
        public const string cWidth = "Width";
        public const string cHeight = "Height";
        public const string cPicPath = "PicPath";
        public const string cHotSpotPicPath = "HotSpotPicPath";
        public const string cColor = "Color";
        public const string cAction = "Action";
        public const string cOpenCloseButtonHeight = "OpenCloseButtonHeight";

        // MC Alignments
        public const string cLeft = "Left";
        public const string cRight = "Right";
        public const string cTop = "Top";
        public const string cBottom = "Bottom";

        // MC Actions
        public const string cSettings = "Settings";
        public const string cOpenClose = "OpenClose";
        //public const string cOpenClose1 = "OpenClose1";
        //public const string cOpenClose2 = "OpenClose2";
        //public const string cOpenClose3 = "OpenClose3";
        //public const string cOpenClose4 = "OpenClose4";
        //public const string cOpenClose5 = "OpenClose5";
        //public const string cOpenClose6 = "OpenClose6";
        //public const string cOpenClose7 = "OpenClose7";
        //public const string cOpenClose8 = "OpenClose8";
        //public const string cOpenClose9 = "OpenClose9";
        //public const string cOpenClose0 = "OpenClose0";
        public const string cNextVessel = "NextVessel";
        public const string cPrevVessel = "PrevVessel";
        public const string cThrottleMax = "ThrottleMax";
        public const string cThrottleUp = "ThrottleUp";
        public const string cThrottleDown = "ThrottleDown";
        public const string cThrottleOff = "ThrottleOff";
        public const string cRCS = "RCS";
        public const string cSAS = "SAS";
        public const string cPitchUp = "PitchUp";
        public const string cPitchDown = "PitchDown";
        public const string cYawRight = "YawRight";
        public const string cYawLeft = "YawLeft";
        public const string cRollRight = "RollRight";
        public const string cRollLeft = "RollLeft";
        public const string cNextStage = "NextStage";
        public const string cActiongroup0 = "Actiongroup0";
        public const string cActiongroup1 = "Actiongroup1";
        public const string cActiongroup2 = "Actiongroup2";
        public const string cActiongroup3 = "Actiongroup3";
        public const string cActiongroup4 = "Actiongroup4";
        public const string cActiongroup5 = "Actiongroup5";
        public const string cActiongroup6 = "Actiongroup6";
        public const string cActiongroup7 = "Actiongroup7";
        public const string cActiongroup8 = "Actiongroup8";
        public const string cActiongroup9 = "Actiongroup9";
        public const string cAnalogCenter = "AnalogCenter";
        public const string cAnalogUp = "AnalogUp";
        public const string cAnalogDown = "AnalogDown";
        public const string cAnalogLeft = "AnalogLeft";
        public const string cAnalogRight = "AnalogRight";
        public const string cAnalogUpLeft = "AnalogUpLeft";
        public const string cAnalogUpRight = "AnalogUpRight";
        public const string cAnalogDownLeft = "AnalogDownLeft";
        public const string cAnalogDownRight = "AnalogDownRight";
        public const string cRCSPitchUp = "RCSPitchUp";
        public const string cRCSPitchDown = "RCSPitchDown";
        public const string cRCSYawRight = "RCSYawRight";
        public const string cRCSYawLeft = "RCSYawLeft";
        public const string cRCSRollRight = "RCSRollRight";
        public const string cRCSRollLeft = "RCSRollLeft";
        public const string cRCSAnalogCenter = "RCSAnalogCenter";
        public const string cRCSAnalogUp = "RCSAnalogUp";
        public const string cRCSAnalogDown = "RCSAnalogDown";
        public const string cRCSAnalogLeft = "RCSAnalogLeft";
        public const string cRCSAnalogRight = "RCSAnalogRight";
        public const string cRCSAnalogUpLeft = "RCSAnalogUpLeft";
        public const string cRCSAnalogUpRight = "RCSAnalogUpRight";
        public const string cRCSAnalogDownLeft = "RCSAnalogDownLeft";
        public const string cRCSAnalogDownRight = "RCSAnalogDownRight";

        #endregion
    }

    public class MCColorAction
    {
        public string Name { get; set; }
        public MCActions Action { get; set; }
        public Color Color { get; set; }


        public override string ToString()
        {
            return string.Format("({0},{1},{2},{3})", Color.r, Color.g, Color.b, Color.a);
        }
    }

    public class MCSize
    {
        public int Width { get; set; }
        public int Height { get; set; }
        public bool IsEmpty
        {
            get
            {
                return (Width == 0 && Height == 0);
            }
        }


        public MCSize(int width = 0, int height = 0)
        {
            Width = width;
            Height = height;
        }


        public override string ToString()
        {
            return string.Format("({0},{1})", Width, Height);
        }
    }

    public class MCMargin
    {
        public int Left { get; set; }
        public int Right { get; set; }
        public int Top { get; set; }
        public int Bottom { get; set; }
        public bool IsEmpty
        {
            get
            {
                return (Left == 0 && Right == 0 && Top == 0 && Bottom == 0);
            }
        }


        public MCMargin(int left = 0, int right = 0, int top = 0, int bottom = 0)
        {
            Left = left;
            Right = right;
            Top = top;
            Bottom = bottom;
        }


        public override string ToString()
        {
            return string.Format("({0},{1},{2},{3})", Left, Right, Top, Right);
        }
    }
}
