﻿using System;
using System.Xml;
using UnityEngine;

namespace MouseController
{
    [KSPAddon(KSPAddon.Startup.Flight, false)]
    public class MouseController : MonoBehaviour
    {
        #region Member

        MCInterface mMCInterface = null;

        // paths
        static string mRoot = KSPUtil.ApplicationRootPath.Replace("\\", "/");
        static string mPluginFolder = mRoot + "GameData/MouseController";
        static string mSettingsPath = mPluginFolder + "/Data/MouseController_Settings.xml";

        // last mouse position
        Vector3 mMousePosition = new Vector3();

        // last window positions
        Rect mDebugWindowPosition = new Rect(50, 0, 350, 500);

        #endregion

        #region public

        bool awakeOnce = false;
        // called on load of .dll
        public void Awake()
        {
            if (!awakeOnce)
                print("=========================> Awake");

            LoadSettings(mSettingsPath.Replace("/", "\\"));

            RenderingManager.AddToPostDrawQueue(10, Update);

            if (!awakeOnce)
                print("=========================> Awake done");
            awakeOnce = true;
        }

        bool startOnce = false;
        // called on scene start
        public void Start()
        {
            if (!updateOnce)
                print("========> Start <========");

            mMCInterface.CalculateMainInterfacePos();
            foreach (var inter in mMCInterface.Subinterfaces)
                inter.Value.CalculateChildInterfacePos();

            // Load textures 
            mMCInterface.TextureMC_Interface = LoadTexture(mMCInterface.PicPath, mMCInterface.Size);
            mMCInterface.TextureMC_Interface_HotSpots = LoadTexture(mMCInterface.HotSpotPicPath, mMCInterface.Size);

            foreach (var entry in mMCInterface.Subinterfaces)
            {
                entry.Value.TextureMC_Interface = LoadTexture(entry.Value.PicPath, mMCInterface.Size);
                entry.Value.TextureMC_Interface_HotSpots = LoadTexture(entry.Value.HotSpotPicPath, mMCInterface.Size);
            }

            // hook to vessel control
            FlightGlobals.ActiveVessel.OnFlyByWire += UpdateControl;

            if (!startOnce)
            {
                System.Text.StringBuilder log = new System.Text.StringBuilder();
                foreach (string str in mMCInterface.PropertyAsStrings)
                    log.AppendLine(str);
                print(log.ToString());
                print("======> Start End <======");
                startOnce = true;
            }
        }

        bool updateOnce = false;
        // called every frame
        public void Update()
        {
            if (!updateOnce)
                print("========> Update <========");

            // get/adjust mouse position (0, 0 = bottom left corner)
            mMousePosition = Input.mousePosition;
            mMousePosition.y = Camera.current.GetScreenHeight() - Input.mousePosition.y;

            //mMCInterface.Update();
            //mMCInterface.UpdateSubInterfaces();

            // draw interface
            foreach (var entry in mMCInterface.Subinterfaces)
                if (entry.Value.TextureMC_Interface != null && !mMCInterface.Collapsed)
                    GUI.DrawTexture(entry.Value.InterfacePos, entry.Value.TextureMC_Interface);

            if (mMCInterface.TextureMC_Interface != null)
                GUI.DrawTexture(mMCInterface.InterfacePos, mMCInterface.TextureMC_Interface);

            // draw settings/debug window
            if (mMCInterface.ShowSettingsWindow)
            {
                GUI.skin = HighLogic.Skin;
                mDebugWindowPosition = GUILayout.Window(44835, mDebugWindowPosition, DrawWindow, "MouseController - Debug Window", GUILayout.MinWidth(200));
            }

            if (!updateOnce)
            {
                //foreach (string str in mMCInterface.States.PropertyAsStrings)
                //    print(str);
                print("======> Update End <======");
                updateOnce = true;
            }
        }

        #endregion

        #region private

        private void DrawWindow(int winID)
        {
            switch (winID)
            {
                case 44835: // Debug Window
                    GUILayout.BeginVertical();
                    //GUILayout.Label("Root = " + Root);
                    //GUILayout.Label("PluginFolder = " + PluginFolder);
                    //GUILayout.Label("ButtonPicPath = " + string.Format("file://{0}/Data/MouseController_Button.png", PluginFolder));
                    //GUILayout.Label("InterfacePicPath = " + string.Format("file://{0}/Data/MouseController_Interface.jpg", PluginFolder));
                    //GUILayout.Label("Screen size = (" + mScreenWidth + ", " + mScreenHeight + ")");
                    //GUILayout.Label("MousePosition = (" + mMousePosition.x + ", " + mMousePosition.y + ")");
                    //GUILayout.Label("WindowPosition = (" + mWindowPosition.x + ", " + mWindowPosition.y + ", " + mWindowPosition.width + ", " + mWindowPosition.height + ")");
                    //GUILayout.Label("WindowPosition.Contains = (" + mWindowPosition.Contains(mMousePosition).ToString() + ")");
                    //GUILayout.Label("L Mouse Button = (" + Input.GetMouseButton(0).ToString() + ")");
                    //GUILayout.Label("Rel. Pic Pos = (" + ((int)mMousePosition.x - (int)mWindowPosition.x).ToString() + ", " + ((int)mMousePosition.y - (int)mWindowPosition.y).ToString() + ")");
                    GUILayout.Label("Texture Pos = " + mMCInterface.States.TexCoords.ToString());
                    GUILayout.Label("Color = " + mMCInterface.States.Color.ToString());
                    GUILayout.Label("InterfaceStates:");
                    GUILayout.Label("HasInput = " + mMCInterface.States.HasInput.ToString());
                    GUILayout.Label("OpenClose = " + mMCInterface.States.OpenCloseDown.ToString());
                    //GUILayout.Label("Func0 = " + mMCInterface.CurrentInterfaceStates.Func0Down.ToString());
                    //GUILayout.Label("Func1 = " + mMCInterface.CurrentInterfaceStates.Func1Down.ToString());
                    //GUILayout.Label("Func2 = " + mMCInterface.CurrentInterfaceStates.Func2Down.ToString());
                    //GUILayout.Label("Func3 = " + mMCInterface.CurrentInterfaceStates.Func3Down.ToString());
                    GUILayout.Label("Yaw = " + mMCInterface.States.Yaw.ToString());
                    GUILayout.Label("Pitch = " + mMCInterface.States.Pitch.ToString());
                    GUILayout.Label("Roll = " + mMCInterface. States.Roll.ToString());
                    GUILayout.Label("RCS = " + mMCInterface.States.RCSDown.ToString());
                    GUILayout.Label("SAS = " + mMCInterface.States.SASDown.ToString());
                    GUILayout.Label("NextStage = " + mMCInterface.States.NextStageDown.ToString());
                    GUILayout.Label("ThrottleFast = " + mMCInterface.States.ThrottleMax.ToString());
                    GUILayout.Label("Throttle = " + mMCInterface.States.Throttle.ToString());
                    GUILayout.Label("ThrottleOff = " + mMCInterface.States.ThrottleOff.ToString());
                    foreach (var entry in mMCInterface.Subinterfaces)
                    {
                        GUILayout.Label("Sub InterfaceStates (" + entry.Value.Name + "):");
                        GUILayout.Label("Texture Pos = " + entry.Value.States.TexCoords.ToString());
                        GUILayout.Label("Color = " + entry.Value.States.Color.ToString());
                        GUILayout.Label("Sub Arrow = " + entry.Value.States.OpenCloseDown.ToString());
                    }
                    //GUILayout.Label("Actiongroup0 = " + mMCInterface.CurrentInterfaceStates.ActionGroup0Down.ToString());
                    //GUILayout.Label("Actiongroup1 = " + mMCInterface.CurrentInterfaceStates.ActionGroup1Down.ToString());
                    //GUILayout.Label("Actiongroup2 = " + mMCInterface.CurrentInterfaceStates.ActionGroup2Down.ToString());
                    //GUILayout.Label("Actiongroup3 = " + mMCInterface.CurrentInterfaceStates.ActionGroup3Down.ToString());
                    //GUILayout.Label("Actiongroup4 = " + mMCInterface.CurrentInterfaceStates.ActionGroup4Down.ToString());
                    //GUILayout.Label("Actiongroup5 = " + mMCInterface.CurrentInterfaceStates.ActionGroup5Down.ToString());
                    //GUILayout.Label("Actiongroup6 = " + mMCInterface.CurrentInterfaceStates.ActionGroup6Down.ToString());
                    //GUILayout.Label("Actiongroup7 = " + mMCInterface.CurrentInterfaceStates.ActionGroup7Down.ToString());
                    //GUILayout.Label("Actiongroup8 = " + mMCInterface.CurrentInterfaceStates.ActionGroup8Down.ToString());
                    //GUILayout.Label("Actiongroup9 = " + mMCInterface.CurrentInterfaceStates.ActionGroup9Down.ToString());
                    GUILayout.EndVertical();

                    GUI.DragWindow();

                    break;
            }
        }

        bool updateControlOnce = false;
        private void UpdateControl(FlightCtrlState flightCtrlState)
        {
            if (!updateControlOnce)
                print("========> UpdateControl");
            
            UpdateInterfaceState();

            // expand/collapse interface
            mMCInterface.Collapsed = !mMCInterface.States.OpenCloseToggled;

            foreach (var entry in mMCInterface.Subinterfaces)
                entry.Value.Collapsed = !entry.Value.States.OpenCloseToggled;

            // activate SAS override on yaw/pitch/roll input.
            if (mMCInterface.States.HasInput)
                FlightGlobals.ActiveVessel.VesselSAS.ManualOverride(true);
            else
                FlightGlobals.ActiveVessel.VesselSAS.ManualOverride(false);

            // next stage
            if (mMCInterface.States.NextStageToggled && Staging.StageCount > 0)
                Staging.ActivateNextStage();

            // show/hide debug window.
            mMCInterface.ShowSettingsWindow = mMCInterface.States.SettingsToggled;

            // de/activate RCS
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.RCS, mMCInterface.States.RCSToggled);

            // de/activate SAS
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.SAS, mMCInterface.States.SASToggled);

            // set action group states.
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom01, mMCInterface.States.ActionGroup0Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom02, mMCInterface.States.ActionGroup1Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom03, mMCInterface.States.ActionGroup2Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom04, mMCInterface.States.ActionGroup3Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom05, mMCInterface.States.ActionGroup4Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom06, mMCInterface.States.ActionGroup5Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom07, mMCInterface.States.ActionGroup6Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom08, mMCInterface.States.ActionGroup7Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom09, mMCInterface.States.ActionGroup8Toggled);
            FlightGlobals.ActiveVessel.ActionGroups.SetGroup(KSPActionGroup.Custom10, mMCInterface.States.ActionGroup9Toggled);

            UpdateThrottle();

            UpdateYawPitchRoll(flightCtrlState);
            UpdateRCS(flightCtrlState);

            if (!updateControlOnce)
                print("========> UpdateControl done");
            updateControlOnce = true;
        }

        private void UpdateInterfaceState()
        {
            // get mouse position on hot spot texture.
            mMCInterface.UpdateTexCoords(mMousePosition);

            foreach (var entry in mMCInterface.Subinterfaces)
                entry.Value.UpdateTexCoords(mMousePosition);

            Color color = mMCInterface.States.Color;
            bool lbutton = Input.GetMouseButton(0);
            mMCInterface.States.AnalogInput = false;
            mMCInterface.States.RCSAnalogInput = false;
            mMCInterface.States.ThrottleMax = MCDirections.None;
            mMCInterface.States.Throttle = MCDirections.None;
            mMCInterface.States.ThrottleOff = false;
            mMCInterface.States.Yaw = MCDirections.None;
            mMCInterface.States.Pitch = MCDirections.None;
            mMCInterface.States.Roll = MCDirections.None;
            mMCInterface.States.RCSYaw = MCDirections.None;
            mMCInterface.States.RCSPitch = MCDirections.None;
            mMCInterface.States.RCSRoll = MCDirections.None;
            foreach (MCColorAction entry in mMCInterface.ColorActionMapping)
            {
                switch (entry.Action)
                {
                    #region Settings

                    case MCActions.Settings:
                        mMCInterface.States.SettingsDown = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    #endregion

                    #region RCS

                    case MCActions.RCS:
                        mMCInterface.States.RCSDown = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    #endregion

                    #region SAS

                    case MCActions.SAS:
                        mMCInterface.States.SASDown = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    #endregion

                    #region Next Stage

                    case MCActions.NextStage:
                        mMCInterface.States.NextStageDown = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    #endregion 

                    #region Vessel change

                    case MCActions.NextVessel:
                        break;

                    case MCActions.PrevVessel:
                        break;

                    #endregion

                    #region Open/Close

                    case MCActions.OpenClose:
                        if (entry.Name == mMCInterface.Name)
                            mMCInterface.States.OpenCloseDown = IsSameColor(color, entry.Color) && lbutton;

                        else
                        {
                            foreach (MCInterface subInterface in mMCInterface.Subinterfaces.Values)
                            {
                                if (entry.Name == subInterface.Name)
                                    subInterface.States.OpenCloseDown = IsSameColor(color, entry.Color) && lbutton;
                            }
                        }
                        break;

                    #endregion

                    #region Throttle

                    case MCActions.ThrottleMax:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.ThrottleMax = MCDirections.Up;
                        break;

                    case MCActions.ThrottleUp:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Throttle = MCDirections.Up;
                        break;

                    case MCActions.ThrottleDown:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Throttle = MCDirections.Down;
                        break;

                    case MCActions.ThrottleOff:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.ThrottleOff = true;
                        break;

                    #endregion

                    #region Pitch, Yaw, Roll

                    case MCActions.PitchUp: // up
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Pitch = MCDirections.Up;
                        break;

                    case MCActions.PitchDown: // down
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Pitch = MCDirections.Down;
                        break;

                    case MCActions.YawRight: // right
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Yaw = MCDirections.Right;
                        break;

                    case MCActions.YawLeft: // left
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Yaw = MCDirections.Left;
                        break;

                    case MCActions.RollRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Roll = MCDirections.Right;
                        break;

                    case MCActions.RollLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.Roll = MCDirections.Left;
                        break;

                    #endregion

                    #region Analog

                    case MCActions.AnalogCenter:
                        break;

                    case MCActions.AnalogUp:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Pitch = MCDirections.Up;
                        }
                        break;

                    case MCActions.AnalogDown:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Pitch = MCDirections.Down;
                        }
                        break;

                    case MCActions.AnalogLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Yaw = MCDirections.Left;
                        }
                        break;

                    case MCActions.AnalogRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Yaw = MCDirections.Right;
                        }
                        break;

                    case MCActions.AnalogUpLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Pitch = MCDirections.Up;
                            mMCInterface.States.Yaw = MCDirections.Left;
                        }
                        break;

                    case MCActions.AnalogUpRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Pitch = MCDirections.Up;
                            mMCInterface.States.Yaw = MCDirections.Right;
                        }
                        break;

                    case MCActions.AnalogDownLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Pitch = MCDirections.Down;
                            mMCInterface.States.Yaw = MCDirections.Left;
                        }
                        break;

                    case MCActions.AnalogDownRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.AnalogInput = true;
                            mMCInterface.States.Pitch = MCDirections.Down;
                            mMCInterface.States.Yaw = MCDirections.Right;
                        }
                        break;

                    #endregion

                    #region RCS Pitch, Yaw, Roll

                    case MCActions.RCSPitchUp: // up
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.RCSPitch = MCDirections.Up;
                        break;

                    case MCActions.RCSPitchDown: // down
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.RCSPitch = MCDirections.Down;
                        break;

                    case MCActions.RCSYawRight: // right
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.RCSYaw = MCDirections.Right;
                        break;

                    case MCActions.RCSYawLeft: // left
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.RCSYaw = MCDirections.Left;
                        break;

                    case MCActions.RCSRollRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.RCSRoll = MCDirections.Right;
                        break;

                    case MCActions.RCSRollLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                            mMCInterface.States.RCSRoll = MCDirections.Left;
                        break;

                    #endregion

                    #region RCS Analog

                    case MCActions.RCSAnalogCenter:
                        break;

                    case MCActions.RCSAnalogUp:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSPitch = MCDirections.Up;
                        }
                        break;

                    case MCActions.RCSAnalogDown:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSPitch = MCDirections.Down;
                        }
                        break;

                    case MCActions.RCSAnalogLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSYaw = MCDirections.Left;
                        }
                        break;

                    case MCActions.RCSAnalogRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSYaw = MCDirections.Right;
                        }
                        break;

                    case MCActions.RCSAnalogUpLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSPitch = MCDirections.Up;
                            mMCInterface.States.RCSYaw = MCDirections.Left;
                        }
                        break;

                    case MCActions.RCSAnalogUpRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSPitch = MCDirections.Up;
                            mMCInterface.States.RCSYaw = MCDirections.Right;
                        }
                        break;

                    case MCActions.RCSAnalogDownLeft:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSPitch = MCDirections.Down;
                            mMCInterface.States.RCSYaw = MCDirections.Left;
                        }
                        break;

                    case MCActions.RCSAnalogDownRight:
                        if (IsSameColor(mMCInterface, entry.Color) && lbutton)
                        {
                            mMCInterface.States.RCSAnalogInput = true;
                            mMCInterface.States.RCSPitch = MCDirections.Down;
                            mMCInterface.States.RCSYaw = MCDirections.Right;
                        }
                        break;

                    #endregion

                    #region ActionGroups

                    case MCActions.Actiongroup0:
                        mMCInterface.States.ActionGroup0Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup1:
                        mMCInterface.States.ActionGroup1Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup2:
                        mMCInterface.States.ActionGroup2Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup3:
                        mMCInterface.States.ActionGroup3Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup4:
                        mMCInterface.States.ActionGroup4Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup5:
                        mMCInterface.States.ActionGroup5Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup6:
                        mMCInterface.States.ActionGroup6Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup7:
                        mMCInterface.States.ActionGroup7Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup8:
                        mMCInterface.States.ActionGroup8Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Actiongroup9:
                        mMCInterface.States.ActionGroup9Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    #endregion

                    #region Func

                    case MCActions.Func0:
                        mMCInterface.States.Func0Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Func1:
                        mMCInterface.States.Func1Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Func2:
                        mMCInterface.States.Func2Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    case MCActions.Func3:
                        mMCInterface.States.Func3Down = IsSameColor(mMCInterface, entry.Color) && lbutton;
                        break;

                    #endregion
                }
            }
        }
        
        private void UpdateThrottle()
        {
            // adjust throttle to input.
            if (mMCInterface.States.ThrottleMax == MCDirections.Up)
                FlightInputHandler.state.mainThrottle = 1f;

            else if (mMCInterface.States.Throttle == MCDirections.Up)
                FlightInputHandler.state.mainThrottle += 0.01f;

            else if (mMCInterface.States.Throttle == MCDirections.Down)
                FlightInputHandler.state.mainThrottle -= 0.01f;

            else if (mMCInterface.States.ThrottleOff)
                FlightInputHandler.state.mainThrottle = 0f;
        }

        private void UpdateYawPitchRoll(FlightCtrlState flightCtrlState)
        {
            // adjust controls to yaw/pitch/roll input
            if (mMCInterface.States.Pitch == MCDirections.Up)
                flightCtrlState.pitch = 1f;
            else if (mMCInterface.States.Pitch == MCDirections.Down)
                flightCtrlState.pitch = -1f;

            if (mMCInterface.States.Yaw == MCDirections.Right)
                flightCtrlState.yaw = 1f;
            else if (mMCInterface.States.Yaw == MCDirections.Left)
                flightCtrlState.yaw = -1f;

            if (mMCInterface.States.Roll == MCDirections.Right)
                flightCtrlState.roll = 1f;
            else if (mMCInterface.States.Roll == MCDirections.Left)
                flightCtrlState.roll = -1f;
        }

        private void UpdateRCS(FlightCtrlState flightCtrlState)
        {
            // adjust controls to RCS input
            if (mMCInterface.States.RCSPitch == MCDirections.Up)
                flightCtrlState.X = -1f;
            if (mMCInterface.States.RCSPitch == MCDirections.Down)
                flightCtrlState.X = 1f;

            if (mMCInterface.States.RCSYaw == MCDirections.Right)
                flightCtrlState.Y = 1f;
            if (mMCInterface.States.RCSYaw == MCDirections.Left)
                flightCtrlState.Y = -1f;

            if (mMCInterface.States.RCSRoll == MCDirections.Right)
                flightCtrlState.Z = 1f;
            if (mMCInterface.States.RCSRoll == MCDirections.Left)
                flightCtrlState.Z = -1f;
        }


        private bool IsSameColor(Color color1, Color color2)
        {
            return (color1.r == color2.r && color1.g == color2.g && color1.b == color2.b); // && color1.a == color2.a);
        }

        private bool IsSameColor(MCInterface mcInterface, Color color2)
        {
            foreach (var entry in mcInterface.Subinterfaces)
            {
                if (entry.Value.States.Color.r == color2.r && entry.Value.States.Color.g == color2.g && entry.Value.States.Color.b == color2.b)
                    return true;
            }

            return (mcInterface.States.Color.r == color2.r && mcInterface.States.Color.g == color2.g && mcInterface.States.Color.b == color2.b); // && color1.a == color2.a);
        }


        private Texture2D LoadTexture(string path, MCSize size)
        {
            Texture2D texture = new Texture2D(size.Width, size.Height);
            WWW wwwP = new WWW("file://" + path);
            wwwP.LoadImageIntoTexture(texture);

            return texture;
        }


        #region LoadSettings

        public void LoadSettings(string path)
        {
            XmlDocument doc = new XmlDocument();
            try
            {
                doc.Load(path.Replace("/", "\\"));
                var i = doc.GetElementsByTagName(Consts.cInterface);
                foreach (XmlNode node in i)
                {
                    if (node.Name == Consts.cInterface)
                    {
                        // read interface
                        try
                        {
                            mMCInterface = GetInterface(node);

                            // read sub interfaces
                            foreach (XmlNode child in node.ChildNodes)
                            {
                                if (child.Name == Consts.cSubInterface)
                                {
                                    MCInterface newSubinterface = GetInterface(child);
                                    if (!newSubinterface.IsEmpty)
                                    {
                                        newSubinterface.Parent = mMCInterface;
                                        mMCInterface.Subinterfaces.Add(newSubinterface.Name, newSubinterface);
                                    }
                                }
                                else if (child.Name == Consts.cColorActionMapping)
                                {
                                    if (mMCInterface == null)
                                        continue;

                                    // read color action mapping
                                    foreach (XmlNode colorActionNode in child.ChildNodes)
                                    {
                                        if (colorActionNode.Name == Consts.cColorAction)
                                        {
                                            MCColorAction colorAction = new MCColorAction();
                                            foreach (XmlAttribute att in colorActionNode.Attributes)
                                            {
                                                if (att.Name == Consts.cName)
                                                    colorAction.Name = att.Value.ToString();

                                                else if (att.Name == Consts.cColor)
                                                    colorAction.Color = GetColor(att.Value.ToString().Trim());

                                                else if (att.Name == Consts.cAction)
                                                    colorAction.Action = GetAction(att.Value.ToString().Trim());
                                            }

                                            mMCInterface.ColorActionMapping.Add(colorAction);
                                        }
                                    }
                                }
                            }
                        }
                        catch (Exception ex)
                        {
                            print("ErrorMessage: " + ex.Message);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                print("ErrorMessage: " + ex.Message);
            }
        }

        private MCInterface GetInterface(XmlNode node)
        {
            MCInterface newInterface = new MCInterface();

            string posX = string.Empty;
            string posY = string.Empty;
            MCSize size = new MCSize();

            foreach (XmlAttribute att in node.Attributes)
            {
                if (att.Name == Consts.cName)
                    newInterface.Name = att.Value.ToString();

                else if (att.Name == Consts.cAlignment)
                    newInterface.Alignment = GetAlignment(att.Value.ToString().Trim());

                else if (att.Name == Consts.cMargin)
                    newInterface.Margin = GetMargin(att.Value.ToString().Trim());

                else if (att.Name == Consts.cX)
                    posX = att.Value;

                else if (att.Name == Consts.cY)
                    posY = att.Value;

                else if (att.Name == Consts.cOpenCloseButtonHeight)
                    newInterface.OpenCloseBtnHeight = int.Parse(att.Value);

                else if (att.Name == Consts.cWidth)
                    size.Width = int.Parse(att.Value);

                else if (att.Name == Consts.cHeight)
                    size.Height = int.Parse(att.Value);

                else if (att.Name == Consts.cPicPath)
                    newInterface.PicPath = mPluginFolder + "/" + att.Value.ToString();

                else if (att.Name == Consts.cHotSpotPicPath)
                    newInterface.HotSpotPicPath = mPluginFolder + "/" + att.Value.ToString();
            }

            if (posX.Contains("%"))
                newInterface.RelativeX = true;
            if (posY.Contains("%"))
                newInterface.RelativeY = true;

            Vector2 pos = new Vector2();
            if (!string.IsNullOrEmpty(posX))
                pos.x = ParseInt(posX);
            if (!string.IsNullOrEmpty(posY))
                pos.y = ParseInt(posY);

            newInterface.Position = pos;
            newInterface.Size = size;

            return newInterface;
        }

        private Color GetColor(string colorAsString)
        {
            int r = 0;
            int g = 0;
            int b = 0;

            string[] temp = colorAsString.Split(',');

            if (temp == null || temp.Length != 3)
            {
                //print("MouseController Error: Can't parse Color");
            }
            else
            {
                try
                {
                    r = int.Parse(temp[0].Trim());
                    g = int.Parse(temp[1].Trim());
                    b = int.Parse(temp[2].Trim());
                }
                catch
                {
                    //print("MouseController Error: Can't parse Color");
                }
            }

            return new Color(r, g, b);
        }

        private MCActions GetAction(string actionAsString)
        {
            MCActions action = MCActions.None;

            switch (actionAsString)
            {
                case Consts.cSettings:
                    action = MCActions.Settings;
                    break;
                case Consts.cOpenClose:
                    action = MCActions.OpenClose;
                    break;
                case Consts.cNextVessel:
                    action = MCActions.NextVessel;
                    break;
                case Consts.cPrevVessel:
                    action = MCActions.PrevVessel;
                    break;
                case Consts.cThrottleMax:
                    action = MCActions.ThrottleMax;
                    break;
                case Consts.cThrottleUp:
                    action = MCActions.ThrottleUp;
                    break;
                case Consts.cThrottleDown:
                    action = MCActions.ThrottleDown;
                    break;
                case Consts.cThrottleOff:
                    action = MCActions.ThrottleOff;
                    break;
                case Consts.cRCS:
                    action = MCActions.RCS;
                    break;
                case Consts.cSAS:
                    action = MCActions.SAS;
                    break;
                case Consts.cPitchUp:
                    action = MCActions.PitchUp;
                    break;
                case Consts.cPitchDown:
                    action = MCActions.PitchDown;
                    break;
                case Consts.cYawRight:
                    action = MCActions.YawRight;
                    break;
                case Consts.cYawLeft:
                    action = MCActions.YawLeft;
                    break;
                case Consts.cRollRight:
                    action = MCActions.RollRight;
                    break;
                case Consts.cRollLeft:
                    action = MCActions.RollLeft;
                    break;
                case Consts.cNextStage:
                    action = MCActions.NextStage;
                    break;
                case Consts.cActiongroup0:
                    action = MCActions.Actiongroup0;
                    break;
                case Consts.cActiongroup1:
                    action = MCActions.Actiongroup1;
                    break;
                case Consts.cActiongroup2:
                    action = MCActions.Actiongroup2;
                    break;
                case Consts.cActiongroup3:
                    action = MCActions.Actiongroup3;
                    break;
                case Consts.cActiongroup4:
                    action = MCActions.Actiongroup4;
                    break;
                case Consts.cActiongroup5:
                    action = MCActions.Actiongroup5;
                    break;
                case Consts.cActiongroup6:
                    action = MCActions.Actiongroup6;
                    break;
                case Consts.cActiongroup7:
                    action = MCActions.Actiongroup7;
                    break;
                case Consts.cActiongroup8:
                    action = MCActions.Actiongroup8;
                    break;
                case Consts.cActiongroup9:
                    action = MCActions.Actiongroup9;
                    break;
                case Consts.cAnalogCenter:
                    action = MCActions.AnalogCenter;
                    break;
                case Consts.cAnalogUp:
                    action = MCActions.AnalogUp;
                    break;
                case Consts.cAnalogDown:
                    action = MCActions.AnalogDown;
                    break;
                case Consts.cAnalogLeft:
                    action = MCActions.AnalogLeft;
                    break;
                case Consts.cAnalogRight:
                    action = MCActions.AnalogRight;
                    break;
                case Consts.cAnalogUpLeft:
                    action = MCActions.AnalogUpLeft;
                    break;
                case Consts.cAnalogUpRight:
                    action = MCActions.AnalogUpRight;
                    break;
                case Consts.cAnalogDownLeft:
                    action = MCActions.AnalogDownLeft;
                    break;
                case Consts.cAnalogDownRight:
                    action = MCActions.AnalogDownRight;
                    break;
                case Consts.cRCSPitchUp:
                    action = MCActions.RCSPitchUp;
                    break;
                case Consts.cRCSPitchDown:
                    action = MCActions.RCSPitchDown;
                    break;
                case Consts.cRCSYawRight:
                    action = MCActions.RCSYawRight;
                    break;
                case Consts.cRCSYawLeft:
                    action = MCActions.RCSYawLeft;
                    break;
                case Consts.cRCSRollRight:
                    action = MCActions.RCSRollRight;
                    break;
                case Consts.cRCSRollLeft:
                    action = MCActions.RCSRollLeft;
                    break;
                case Consts.cRCSAnalogCenter:
                    action = MCActions.RCSAnalogCenter;
                    break;
                case Consts.cRCSAnalogUp:
                    action = MCActions.RCSAnalogUp;
                    break;
                case Consts.cRCSAnalogDown:
                    action = MCActions.RCSAnalogDown;
                    break;
                case Consts.cRCSAnalogLeft:
                    action = MCActions.RCSAnalogLeft;
                    break;
                case Consts.cRCSAnalogRight:
                    action = MCActions.RCSAnalogRight;
                    break;
                case Consts.cRCSAnalogUpLeft:
                    action = MCActions.RCSAnalogUpLeft;
                    break;
                case Consts.cRCSAnalogUpRight:
                    action = MCActions.RCSAnalogUpRight;
                    break;
                case Consts.cRCSAnalogDownLeft:
                    action = MCActions.RCSAnalogDownLeft;
                    break;
                case Consts.cRCSAnalogDownRight:
                    action = MCActions.RCSAnalogDownRight;
                    break;
                default:
                    action = MCActions.None;
                    break;
            }

            return action;
        }

        private MCAlignment GetAlignment(string alignmentAsString)
        {
            MCAlignment alignment = MCAlignment.Bottom;

            switch (alignmentAsString)
            {
                case Consts.cLeft:
                    alignment = MCAlignment.Left;
                    break;
                case Consts.cRight:
                    alignment = MCAlignment.Right;
                    break;
                case Consts.cTop:
                    alignment = MCAlignment.Top;
                    break;
                case Consts.cBottom:
                    alignment = MCAlignment.Bottom;
                    break;
            }

            return alignment;
        }

        private MCMargin GetMargin(string marginAsString)
        {
            int l = 0;
            int r = 0;
            int t = 0;
            int b = 0;

            string[] temp = marginAsString.Split(',');

            if (temp == null || temp.Length != 4)
            {
                //print("MouseController Error: Can't parse Margin");
            }
            else
            {
                try
                {
                    l = int.Parse(temp[0].Trim());
                    r = int.Parse(temp[1].Trim());
                    t = int.Parse(temp[2].Trim());
                    b = int.Parse(temp[3].Trim());
                }
                catch
                {
                    //print("MouseController Error: Can't parse Margin");
                }
            }

            return new MCMargin(l, r, t, b);
        }

        private int ParseInt(string integer)
        {
            int pos = 0;
            try
            {
                if (integer.Contains("%"))
                    pos = int.Parse(integer.Replace("%", "").Trim());
                else
                    pos = int.Parse(integer);
            }
            catch
            {
                //print("MouseController error: Error during int parsing!");
            }

            return pos;
        }

        #endregion

        #endregion
    }
}
